@@ -9,6 +9,7 @@ from tutumcli import parsers
 from tutumcli import commands
 from tutumcli.exceptions import InternalError
 
+
 sys.stdout = codecs.getwriter('utf8')(sys.stdout)
 
 logging.basicConfig()
@@ -28,6 +29,7 @@ def initialize_parser():
     parsers.add_node_parser(subparsers)
     parsers.add_nodecluster_parser(subparsers)
     parsers.add_service_parser(subparsers)
+    parsers.add_tag_parser(subparsers)
     return parser
 
 
@@ -43,10 +45,10 @@ def patch_help_option(argv=sys.argv):
 
     if len(args) == 1:
         args.append('-h')
-    elif len(args) == 2 and args[1] in ['service', 'build', 'container', 'image', 'node', 'nodecluster']:
+    elif len(args) == 2 and args[1] in ['service', 'build', 'container', 'image', 'node', 'nodecluster', 'tag']:
         args.append('-h')
     elif len(args) == 3:
-        if args[1] == 'service' and args[2] in ['alias', 'inspect', 'logs', 'redeploy', 'run', 'scale', 'set',
+        if args[1] == 'service' and args[2] in ['create', 'inspect', 'logs', 'redeploy', 'run', 'scale', 'set',
                                                 'start', 'stop', 'terminate']:
             args.append('-h')
         elif args[1] == 'container' and args[2] in ['inspect', 'logs', 'redeploy', 'start', 'stop',
@@ -58,6 +60,8 @@ def patch_help_option(argv=sys.argv):
             args.append('-h')
         elif args[1] == 'nodecluster' and args[2] in ['create', 'inspect', 'rm', 'scale']:
             args.append('-h')
+        elif args[1] == 'tag' and args[2] in ['add', 'list', 'rm']:
+            args.append('-h')
     if debug:
         args.insert(1, '--debug')
     return args[1:]
@@ -72,8 +76,16 @@ def dispatch_cmds(args):
     if args.cmd == 'build':
         commands.build(args.tag, args.directory, args.quiet, args.no_cache)
     elif args.cmd == 'service':
-        if args.subcmd == 'alias':
-            commands.service_alias(args.identifier, args.dns)
+        if args.subcmd == 'create':
+            commands.service_create(image=args.image, name=args.name, cpu_shares=args.cpushares,
+                                    memory=args.memory, privileged=args.privileged,
+                                    target_num_containers=args.target_num_containers, run_command=args.run_command,
+                                    entrypoint=args.entrypoint, expose=args.expose, publish=args.publish,
+                                    envvars=args.env,
+                                    tag=args.tag, linked_to_service=args.link_service,
+                                    autorestart=args.autorestart,
+                                    autoreplace=args.autoreplace, autodestroy=args.autodestroy, roles=args.role,
+                                    sequential=args.sequential)
         elif args.subcmd == 'inspect':
             commands.service_inspect(args.identifier)
         elif args.subcmd == 'logs':
@@ -87,7 +99,7 @@ def dispatch_cmds(args):
                                  memory=args.memory, privileged=args.privileged,
                                  target_num_containers=args.target_num_containers, run_command=args.run_command,
                                  entrypoint=args.entrypoint, expose=args.expose, publish=args.publish, envvars=args.env,
-                                 linked_to_service=args.link_service,
+                                 tag=args.tag, linked_to_service=args.link_service,
                                  autorestart=args.autorestart,
                                  autoreplace=args.autoreplace, autodestroy=args.autodestroy, roles=args.role,
                                  sequential=args.sequential)
@@ -153,6 +165,13 @@ def dispatch_cmds(args):
             commands.nodecluster_rm(args.identifier)
         elif args.subcmd == 'scale':
             commands.nodecluster_scale(args.identifier, args.target_num_nodes)
+    elif args.cmd == 'tag':
+        if args.subcmd == 'add':
+            commands.tag_add(args.identifier, args.tag)
+        elif args.subcmd == 'list':
+            commands.tag_list(args.identifier, args.quiet)
+        if args.subcmd == 'rm':
+            commands.tag_rm(args.identifier, args.tag)
 
 
 def main():
